package cn.jyjz.xiaoyao.createcode.createcode.java.javacode;


import cn.jyjz.xiaoyao.createcode.object.Table;
import cn.jyjz.xiaoyao.createcode.util.ForFileUtil;
import java.io.File;
import java.io.IOException;
import lombok.extern.slf4j.Slf4j;

/**
 * 创建数据库的连接
 * Created by zhangyong on 2018/2/8.
 */
@Slf4j
public class CreateDataDao {

    public static final String MAPPER_FILE_NAME = "dataDao";

    public static String getJpaName(String tableJavaName){

        return tableJavaName +  "JpaDao";
    }
    public static String getMapperName(String tableJavaName){
        return tableJavaName +  "MybatisDao";
    }

    public static String getMapperPath(String tableJavaName,String javapath){

        String relativepath = tableJavaName.toLowerCase() + File.separator + MAPPER_FILE_NAME;

        relativepath = ForFileUtil.formatPath(javapath) + File.separator + relativepath;

        return relativepath;
    }

    public static String getMapperXmlPath(String tableJavaName,String javapath){

        String mapperDaoimportpath = CreateDataDao.getMapperPath(tableJavaName,javapath) +  File.separator + getMapperName(tableJavaName);

        mapperDaoimportpath = mapperDaoimportpath.replace(File.separator,".");

        return mapperDaoimportpath;
    }
    public static String getImportJpaPath(String tableJavaName,String javapath){

        String mapperDaoimportpath = CreateDataDao.getMapperPath(tableJavaName,javapath) +  File.separator + getJpaName(tableJavaName);

        mapperDaoimportpath = mapperDaoimportpath.replace(File.separator,".");

        return mapperDaoimportpath;
    }
    /**
     *
     * physicspath：文件实际保存路径
     * table 数据库表对象
     * javapath java代码路径
     * htmlpath html存储路径
     * */
    public static boolean createDao(String physicspath, Table table, String javapath, String htmlpath){

        if(!physicspath.endsWith(File.separator)){
            physicspath = physicspath + File.separator;
        }

//        if(!createJpa( physicspath,  table,javapath)){
//            log.error("【创建jpa】失败");
//        }
        if(!createMapper( physicspath,  table,javapath)){
            log.error("【创建Mapper】失败");
        }
        return true;
    }

    private static boolean createMapper(String physicspath, Table table,String javapath){
        //输出内容
        StringBuffer modeldata = new StringBuffer();

        String relativepath = CreateDataDao.getMapperPath(table.getJavaName(),javapath) ;

        //创建model类的物理路径
        String changemodelpath = physicspath + relativepath + File.separator + getMapperName(table.getJavaName()) + ".java";

        //java引入的model路径
        String modelRelativepath = CreateModel.getRelativepath(table.getJavaName(),javapath).replace(File.separator,".") + "." + table.getJavaName();

        //打包路径
        modeldata.append("package " + relativepath.replace(File.separator,".") + ";\n\n");

        modeldata.append("import " + modelRelativepath + ";\n");
        modeldata.append("import com.baomidou.mybatisplus.core.mapper.BaseMapper;\n");
        modeldata.append("import org.apache.ibatis.annotations.Mapper;\n");


        modeldata.append("/**\n");
        modeldata.append("* 数据持久化\n");
        modeldata.append("*/\n");
        modeldata.append("@Mapper\n");
        modeldata.append("public interface " + getMapperName(table.getJavaName()) + " extends BaseMapper<" + table.getJavaName() + "> {\n");

        modeldata.append("}");



        try {
            return ForFileUtil.writeFileContent(changemodelpath,modeldata.toString());
        } catch (IOException e) {
            e.printStackTrace();
        }
        return false;
    }
}
