package cn.jyjz.xiaoyao.createcode.createcode.java.javacode;

import static java.io.File.separator;

import cn.jyjz.xiaoyao.createcode.enums.JdbcTypeEnum;
import cn.jyjz.xiaoyao.createcode.object.Column;
import cn.jyjz.xiaoyao.createcode.object.Table;
import cn.jyjz.xiaoyao.createcode.util.ForFileUtil;
import java.io.IOException;
import java.util.List;

/**
 * Created by zhangyong on 2018/2/9.
 */
public class CreateMapper {

    public static final String MODEL_FILE_NAME = "Mapper";

    /**
     *
     * physicspath：文件实际保存路径
     * table 数据库表对象
     * htmlpath html存储路径
     * */
    public static boolean createModel(String physicspath, Table table, String mapperpath, String javapath, String htmlpath){
        //输出内容
        StringBuffer modeldata = new StringBuffer();

        if(!physicspath.endsWith(separator)){
            physicspath = physicspath + separator;
        }

        mapperpath = mapperpath.replace("/", separator);
        //去掉路径上第一个分隔符
        if(mapperpath.startsWith(separator)){
            mapperpath = javapath.substring(1,mapperpath.length());
        }
        if(mapperpath.endsWith(separator)){
            mapperpath = javapath.substring(0,mapperpath.length()-1);
        }

        //创建model类的物理路径
        String changemodelpath = physicspath + mapperpath + separator + table.getJavaName() + "Mapper.xml";


        //打包路径
        modeldata.append("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");

        modeldata.append("<!DOCTYPE mapper PUBLIC \"-//mybatis.org//DTD Mapper 3.0//EN\" \"http://mybatis.org/dtd/mybatis-3-mapper.dtd\">\n");

        modeldata.append("<mapper namespace=\"" + CreateDataDao.getMapperXmlPath(table.getJavaName(),javapath) + "\">\n");

        //字段映射
        modeldata.append("\t<resultMap id=\"BaseResultMap\" type=\"" + CreateModel.getRelativepath(table.getJavaName(),javapath).replace(separator,".") + "." + table.getJavaName() + "\">\n");
        for(Column column:table.getColumnList()){
            modeldata.append(formatMapperAttribute(column));
        }
        modeldata.append("\t</resultMap>\n");

        //搜索字段格式化
        modeldata.append("\t<sql id=\"Base_Column_List\">\n");

        StringBuffer columnlist = new StringBuffer();
        for(Column column:table.getColumnList()){
            if(null == columnlist || columnlist.length() < 1){
                columnlist.append(formatMapperBase_Column_List(column));
            }else{
                columnlist.append(",\n" + formatMapperBase_Column_List(column));
            }
        }
        modeldata.append(columnlist.toString());
        modeldata.append("\n");
        modeldata.append("\t</sql>\n\n");

        //搜索条件，针对页面搜索的json格式
        modeldata.append("\t<sql id=\"Base_Where\">\n");
        modeldata.append(formatMapperWhereJson(table.getColumnList()));
        modeldata.append("\t</sql>\n\n");

        modeldata.append("</mapper>");

        try {
            return ForFileUtil.writeFileContent(changemodelpath,modeldata.toString());
        } catch (IOException e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * 字段映射
     * @param column
     * @return
     */
    private static String formatMapperAttribute(Column column){
        StringBuffer returnstring = new StringBuffer();
        if(column.isPk()){
            returnstring.append("\t\t<id column=\"" + column.getColumn() + "\" property=\"" + column.getJavaName() + "\" jdbcType=\"" + JdbcTypeEnum
                .getJdbcType(column.getColumnType().toUpperCase()) + "\"/>\n");

        }else{
            returnstring.append("\t\t<result column=\"" + column.getColumn() + "\" property=\"" + column.getJavaName() + "\" jdbcType=\"" + JdbcTypeEnum.getJdbcType(column.getColumnType().toUpperCase()) + "\"/>\n");

        }

        return returnstring.toString();
    }

    /**
     * 搜索字段格式化
     * @param column
     * @return
     */
    private static String formatMapperBase_Column_List(Column column){

        return ("\t\t" + column.getColumn() + " AS " + column.getJavaName());


    }

    /**
     *
     * 搜索条件，针对页面搜索的json格式
     *
     * */
    private static String formatMapperWhereJson(List<Column> columns){
        StringBuffer returndata = new StringBuffer("\t\t<where>\n");
        returndata.append("\t\t\t<trim prefixOverrides=\"and|or\">\n");
        for(Column column:columns){

            if(column.getColumn().equals("USERID") || column.getColumn().equals("DEPARTID")){
                continue;
            }
            returndata.append("\t\t\t\t");
            returndata.append("<if test=\"" + column.getJavaName() + " != null\"> and ");

            returndata.append(column.getColumn() + " ${" + column.getJavaName() + ".dataOp}");
            returndata.append(" ${" + column.getJavaName() + ".likestar}");
            returndata.append("#{" + column.getJavaName() + ".value}");
            returndata.append("${" + column.getJavaName() + ".likeend}");

            returndata.append("</if>\n");
        }


        returndata.append("\t\t\t\t<if test=\"userid != null\"> and USERID ${userid.dataOp} ${userid.likestar}#{userid.value}${userid.likeend}</if>\n");

        returndata.append("\t\t\t\t<if test=\"listdept != null\">\n");
        returndata.append("\t\t\t\t\tand DEPARTID in\n");
        returndata.append("\t\t\t\t\t<foreach collection=\"listdept.value\" item=\"item\" index=\"index\" open=\"(\" separator=\",\" close=\")\" >\n");
        returndata.append("\t\t\t\t\t\t#{item}\n");
        returndata.append("\t\t\t\t\t</foreach>\n");
        returndata.append("\t\t\t\t</if>\n");


        returndata.append("\t\t\t</trim>\n");
        returndata.append("\t\t</where>\n");

        returndata.append( "\t\t<if test=\"page != null\">\n");
        returndata.append("\t\t\t<if test=\"page.sortname != null\"> order by ${page.sortname} ${page.sortorder}</if> \n");
        returndata.append("\t\t\tlimit ${page.start}, ${page.Pagesize}\n");
        returndata.append("\t\t</if>\n");
        return returndata.toString();
    }

    /**
     * 格式化mapper写入字段映射关系
     * */
    public static String formatMapperUpdate(List<Column> columns){

        //返回的字符串
        StringBuffer returnstring = new StringBuffer();

        for(Column column:columns){
            if(column.isPk()){
                continue;
            }

            returnstring.append("\n\t\t\t<if test=\"" + column.getJavaName() + " != null\">");

            returnstring.append(column.getColumn() + " = ");

            returnstring.append("#{" + column.getJavaName() + ",jdbcType=" + JdbcTypeEnum.getJdbcType(column.getColumnType().toUpperCase()) + "}");

            returnstring.append(",</if>");
        }

        return returnstring.toString();
    }

    /**
     * 格式化mapper写入字段映射关系
     * */
    public static String formatMapperInsert(List<Column> columns){

        String returndata = "";
        String insertData = "\t\t\t";
        String valueData = "";

        int i = 0;
        for(Column column:columns){

            String ccode = column.getColumn();
            if(i == 0){
                insertData = insertData + ccode;
                valueData = valueData + "#{" + column.getJavaName() + ",jdbcType=" + JdbcTypeEnum.getJdbcType(column.getColumnType().toUpperCase()) + "}";
            }else{
                insertData = insertData + ",\n\t\t\t" + ccode;
                valueData = valueData + ",\n\t\t\t" + "#{" + column.getJavaName() + ",jdbcType=" + JdbcTypeEnum.getJdbcType(column.getColumnType().toUpperCase()) + "}";
            }
            i++;
        }

        returndata = insertData + "\n\t\t)\n\t\tVALUES(\n\t\t\t" + valueData + "\n\t\t)\t\t\n";

        return returndata;
    }
}
