package cn.jyjz.xiaoyao.createcode.createcode.java.javacode;


import cn.jyjz.xiaoyao.createcode.object.Column;
import cn.jyjz.xiaoyao.createcode.object.Table;
import cn.jyjz.xiaoyao.createcode.util.ForFileUtil;
import cn.jyjz.xiaoyao.createcode.util.KeyUtil;
import java.io.File;
import java.io.IOException;

/**
 * 创建数据与实体类的映射对象
 * Created by zhangyong on 2018/2/7.
 */
public class CreateModel {

    public static final String MODEL_FILE_NAME = "dataobject";

    public static String getRelativepath(String tableJavaName,String javapath){

        String relativepath = tableJavaName.toLowerCase() + File.separator + MODEL_FILE_NAME;

        relativepath = ForFileUtil.formatPath(javapath) + File.separator + relativepath;

        return relativepath;
    }

    /**
     *
     * physicspath：文件实际保存路径
     * table 数据库表对象
     * javapath java代码路径
     * htmlpath html存储路径
     * */
    public static boolean createModel(String physicspath, Table table, String javapath, String htmlpath){



        if(!physicspath.endsWith(File.separator)){
            physicspath = physicspath + File.separator;
        }

        //输出内容
        StringBuffer modeldata = new StringBuffer();

        String relativepath = getRelativepath(table.getJavaName(),javapath);

        //创建model类的物理路径
        String changemodelpath = physicspath + relativepath + File.separator + table.getJavaName() + ".java";

        //打包路径
        modeldata.append("package " + relativepath.replace(File.separator,".") + ";\n\n");

        modeldata.append("import cn.jyjz.xiaoyao.common.mybatisplus.dto.BaseDto;\n");
        modeldata.append("import cn.jyjz.xiaoyao.common.base.util.requestFormat.SearchQuery;\n\n");

        modeldata.append("import com.baomidou.mybatisplus.annotation.TableField;\n");
        modeldata.append("import com.baomidou.mybatisplus.annotation.IdType;\n");
        modeldata.append("import com.baomidou.mybatisplus.annotation.TableId;\n");
        modeldata.append("import com.baomidou.mybatisplus.annotation.TableName;\n");
        modeldata.append("import com.fasterxml.jackson.databind.annotation.JsonSerialize;\n");
        modeldata.append("import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;\n");
        modeldata.append("import io.swagger.v3.oas.annotations.media.Schema;\n");
        modeldata.append("import lombok.AllArgsConstructor;\n");
        modeldata.append("import lombok.Builder;\n");
        modeldata.append("import lombok.Data;\n");
        modeldata.append("import lombok.NoArgsConstructor;\n");
        modeldata.append("import java.util.Map;\n");
        modeldata.append("import java.util.AbstractMap.SimpleEntry;\n");
        modeldata.append("import java.util.stream.Stream;\n");
        modeldata.append("import java.util.stream.Collectors;\n");
        modeldata.append(" \n");
        modeldata.append(" \n");

        modeldata.append("\n");
        modeldata.append("@Data\n");
        modeldata.append("@Builder\n");
        modeldata.append("@NoArgsConstructor\n");
        modeldata.append("@AllArgsConstructor\n");
        modeldata.append("@Schema(name = \"" + table.getTableName() + "\", title = \"" + table.getTableName() +  "\")\n");
        modeldata.append("@TableName(value = \"" + table.getTableCode() + "\")\n");
        modeldata.append("public class " + table.getJavaName() + " implements BaseDto,java.io.Serializable {\n\n");
        modeldata.append( "\tprivate static final long serialVersionUID = " + KeyUtil.hash() + "L;\n");

        for(Column column:table.getColumnList()){

            modeldata.append(faomatContent(column));
        }

        modeldata.append( "\n");
        modeldata.append( "\t//保存当前登录用户的数据权限范围的搜索条件\n");
        modeldata.append( "\t@TableField(exist = false)\n");
        modeldata.append( "\tprivate SearchQuery searchQueryrolesShowleave;\n");

        modeldata.append( "\n");
        modeldata.append( "\t@TableField(exist = false)" + "\n");
        modeldata.append( "\tprivate final Map<String,String> query = Stream.of(" + "\n");

        modeldata.append( "\t\tnew SimpleEntry<>(\"listdept\",\"deptid\"),\n");

        int tempi = 0;
        for(Column column:table.getColumnList()){

            modeldata.append(faomatMap(column));

            if(tempi < (table.getColumnList().size() - 1)){
                modeldata.append(",\n");
            }else{
                modeldata.append("\n");
            }
            tempi++;
        }
        modeldata.append( "\t)" + "\n");
        modeldata.append( "\t.collect(Collectors.toMap(SimpleEntry::getKey, SimpleEntry::getValue));" + "\n");

        modeldata.append( "\t@Override\n");
        modeldata.append( "\tpublic String getQueryFiled(String filedname){\n");
        modeldata.append( "\t\tString obj = null;\n");
        modeldata.append( "\t\tif(null != query && query.size() > 0){\n");
        modeldata.append( "\t\tobj = query.get(filedname);\n");
        modeldata.append( "\t\t}\n");
        modeldata.append( "\t\treturn obj;\n");
        modeldata.append( "\t}\n");




//        for(Column column:table.getColumnList()){
//
//            modeldata.append(faomatContentBuGetAndSet(column));
//        }

        modeldata.append("}");
        try {
            return ForFileUtil.writeFileContent(changemodelpath,modeldata.toString());
        } catch (IOException e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * 生成类属性
     * */
    private static String faomatContent(Column column){
        StringBuffer returnstring = new StringBuffer();
        //字段说明
        returnstring.append("\t@Schema(description = \"" + column.getColComment() +  "\")\n");

        //主键策略
        if(column.isPk()){
            returnstring.append("\t@TableId(value = \"id\",type = IdType.ASSIGN_ID)\n");
            returnstring.append("\t@JsonSerialize(using = ToStringSerializer.class)\n");
        }
        if (column.getJavaType().equals("java.util.Date")){
            returnstring.append("\t@JsonFormat(pattern = \"yyyy-MM-dd\",timezone=\"GMT+8\")\n");
            returnstring.append("\t@DateTimeFormat(pattern = \"yyyy-MM-dd\")\n");
        }

        String length = "";
        if(null != column.getLength()){

            if(null != column.getScale()){
                length = ",precision=" + column.getLength() + ",scale=" + column.getScale();
            }else{
                length = ",length=" + column.getLength();
            }
        }

        //字段映射
        returnstring.append("\t@TableField(value=\"" + column.getColumn() + "\")\n");
        if (column.getJavaType().equals("java.util.Date")){
            returnstring.append("\tprivate " + column.getJavaType() + " " + column.getJavaName() + ";\n");
        }else {
            returnstring.append("\tprivate " + column.getJavaType() + " " + column.getJavaName() + ";\n\n");
        }


        return returnstring.toString();
    }

    private static String faomatMap(Column column){

        return "\t\tnew SimpleEntry<>(\"" + column.getColumn() + "\"," + "\"" + column.getJavaName() + "\")";
    }

    /**
     * 生成类属性的get和set方法
     * @param column
     * @return
     */
    private static String faomatContentBuGetAndSet(Column column){

        StringBuffer returnstring = new StringBuffer();

        String javaType = "";
        String jsonFormat = "";
        if(column.getJavaType().equals("long") || column.getJavaType().equals("Long")){
            javaType = "Long";
            jsonFormat = "@JsonSerialize(using = ToStringSerializer.class)";
        }else{
            javaType = column.getJavaType();
        }
        returnstring.append("\t" + jsonFormat + "\n");
        returnstring.append("\tpublic " + javaType + " get" + column.getJavaNameFirstUp() + "(){\n");
        returnstring.append("\t\treturn this." + column.getJavaName() + ";\n");
        returnstring.append("\t}\n\n");

        returnstring.append("\tpublic void set" + column.getJavaNameFirstUp() + "(" + column.getJavaType() + " " + column.getJavaName() + "){\n");
        returnstring.append("\t\tthis." + column.getJavaName() + " = " + column.getJavaName() + ";\n");
        returnstring.append("\t}\n\n");

        return returnstring.toString();

    }
}
