package cn.jyjz.xiaoyao.createcode.createcode.java.javacode;

import cn.jyjz.xiaoyao.createcode.object.Table;
import cn.jyjz.xiaoyao.createcode.util.ForFileUtil;
import java.io.File;
import java.io.IOException;
import lombok.extern.slf4j.Slf4j;

/**
 * 创建处理层的类
 * Created by zhangyong on 2018/2/9.
 */
@Slf4j
public class CreateService {
    public static final String SERVICE_FILE_NAME = "service";
    public static final String SERVICEIMPL_FILE_NAME = "impl";

    public static String getServiceName(String tablename){

        return tablename +  "Service";
    }
    public static String getServiceImplName(String tablename){
        return tablename +  "ServiceImpl";
    }

    public static String getRelativepath(String tableJavaName,String javapath){

        String relativepath = tableJavaName.toLowerCase() + File.separator + SERVICE_FILE_NAME;

        relativepath = ForFileUtil.formatPath(javapath) + File.separator + relativepath;

        return relativepath;
    }

    /**
     *
     * physicspath：文件实际保存路径
     * table 数据库表对象
     * javapath java代码路径
     * htmlpath html存储路径
     * */
    public static boolean createService(String physicspath, Table table, String javapath, String htmlpath){

        if(!physicspath.endsWith(File.separator)){
            physicspath = physicspath + File.separator;
        }

        if(!createServiceInterface( physicspath,  table,javapath)){
            log.error("【创建Service】失败");
        }
        if(!createServiceImpl( physicspath,  table,javapath)){
            log.error("【创建MServiceImpl】失败");
        }
        return true;
    }

    private static boolean createServiceInterface(String physicspath, Table table,String javapath){
        //输出内容
        StringBuffer modeldata = new StringBuffer();

        String relativepath = CreateService.getRelativepath(table.getJavaName(),javapath) ;

        //创建model类的物理路径
        String changemodelpath = physicspath + relativepath + File.separator + getServiceName(table.getJavaName())  + ".java";

        //java引入的model路径
        String modelRelativepath = CreateModel.getRelativepath(table.getJavaName(),javapath).replace(File.separator,".") + "." + table.getJavaName();

        //打包路径
        modeldata.append("package " + relativepath.replace(File.separator,".") + ";\n\n");

        modeldata.append("import " + modelRelativepath + ";\n");

        modeldata.append("import java.util.List;\n");
        modeldata.append("import cn.jyjz.xiaoyao.common.mybatisplus.base.BaseService;\n");

        modeldata.append("/**\n");
        modeldata.append("* 业务层\n");
        modeldata.append("*/\n");
        modeldata.append("public interface " + getServiceName(table.getJavaName()) + " extends BaseService<" + table.getJavaName() + ">  {\n");

        modeldata.append("}");

        try {
            return ForFileUtil.writeFileContent(changemodelpath,modeldata.toString());
        } catch (IOException e) {
            e.printStackTrace();
        }
        return false;
    }

    private static boolean createServiceImpl(String physicspath, Table table,String javapath){
        //输出内容
        StringBuffer modeldata = new StringBuffer();

        //service的路径
        String relativepathService = CreateService.getRelativepath(table.getJavaName(),javapath) ;
        relativepathService = relativepathService.replace(File.separator,".") + "." + getServiceName(table.getJavaName());
        //service实现的路径
        String relativepath = CreateService.getRelativepath(table.getJavaName(),javapath) + File.separator + SERVICEIMPL_FILE_NAME;

        //创建model类的物理路径
        String changemodelpath = physicspath + relativepath + File.separator + getServiceImplName(table.getJavaName())  + ".java";

        //java引入的model路径
        String modelRelativepath = CreateModel.getRelativepath(table.getJavaName(),javapath).replace(File.separator,".") + "." + table.getJavaName();

        //打包路径
        modeldata.append("package " + relativepath.replace(File.separator,".") + ";\n\n");

        modeldata.append("import " + modelRelativepath + ";\n");
        modeldata.append("import " + relativepathService + ";\n");
        modeldata.append("import " + CreateDataDao.getMapperXmlPath(table.getJavaName(),javapath) + ";\n");


        modeldata.append("import org.springframework.beans.factory.annotation.Autowired;\n");
        modeldata.append("import org.springframework.stereotype.Service;\n");
        modeldata.append("import cn.jyjz.xiaoyao.common.mybatisplus.base.BaseServiceImpl;\n");
        modeldata.append("import lombok.extern.slf4j.Slf4j;\n");
        modeldata.append("import java.util.List;\n");

        modeldata.append("/**\n");
        modeldata.append("* 业务层实现\n");
        modeldata.append("*/\n");

        modeldata.append("@Service\n");
        modeldata.append("@Slf4j\n");
        modeldata.append("public class " + getServiceImplName(table.getJavaName()) + " extends BaseServiceImpl<" + CreateDataDao.getMapperName(table.getJavaName()) + "," + table.getJavaName() + "> " + " implements " + getServiceName(table.getJavaName())  +  " {\n");


        modeldata.append("\t@Autowired\n");
        modeldata.append("\tprivate " + CreateDataDao.getMapperName(table.getJavaName()) + " " + CreateDataDao.getMapperName(table.getJavaName()).toLowerCase() + ";\n");
        modeldata.append("\t\n");


        modeldata.append("}");

        try {
            return ForFileUtil.writeFileContent(changemodelpath,modeldata.toString());
        } catch (IOException e) {
            e.printStackTrace();
        }
        return false;
    }
}
