package cn.jyjz.xiaoyao.createcode.read;

import cn.jyjz.xiaoyao.createcode.createcode.CreateCodeFactory;
import cn.jyjz.xiaoyao.createcode.object.Column;
import cn.jyjz.xiaoyao.createcode.object.Table;
import cn.jyjz.xiaoyao.createcode.util.ForFileUtil;
import com.google.common.base.CaseFormat;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.XPath;
import org.dom4j.io.SAXReader;

public class XmlPrase {

    /**
     * 解析xml文档，将解析到得数据封装到Table对象当中
     */

    public List<Table> getXmlTable(String xmlPath) {

        List<Table> tableList = new ArrayList<Table>();

        //根据xml文件，将文件已对象的形式表现出来
        File file = new File(xmlPath);
        //构建解析方式
        SAXReader reader = new SAXReader();
        //得到文档对象
        Document doc = null;
        try {
            doc = reader.read(file);
        } catch (DocumentException e) {
            return null;
        }
        //根据xpath方式得到所要得到xml文档的具体对象,根据分析解析xml文档可知，xml文档中含有前缀名
        Map<String, String> map = new HashMap<String, String>();
        map.put("c", "collection");
        //根据xml文档，//c:Table 即为得到的文档对象
        XPath path = doc.createXPath("//c:Tables");
        path.setNamespaceURIs(map);
        List<Element> list = path.selectNodes(doc);
        //得到tables对象，该对象是该pdm文件中所有表的集合
        for (Element element : list) {
            for (Iterator<Element> iter = element.elementIterator("Table"); iter.hasNext(); ) {
                Element table_s = iter.next();
                Table table = new Table();
                table.setTableName(table_s.elementText("Name"));
                table.setTableCode(table_s.elementText("Code"));
                table.setJavaName(CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.UPPER_CAMEL, ForFileUtil
                    .tableFormatName(table.getTableCode())));
                table.setJspName(CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.LOWER_CAMEL, ForFileUtil.tableFormatName(table.getTableCode())));
                //通过一个table对象，得到columns对象
                Element columns = table_s.element("Columns");
                List<Column> columnList = new ArrayList<Column>();


                for (Iterator<Element> cols = columns.elementIterator("Column"); cols.hasNext(); ) {
                    Element column = cols.next();
                    Column column2 = new Column();
                    column2.setColumn(column.elementText("Code"));
                    column2.setColumnName(column.elementText("Name"));
                    column2.setColumnType(column.elementText("DataType"));
                    column2.setJavaName(
                        CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.LOWER_CAMEL, column2.getColumn()));
                    column2.setJavaNameFirstUp(
                        CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.UPPER_CAMEL, column2.getColumn()));

                    if (column.elementTextTrim("DataType").indexOf("(") > 0) {
                        column2.setColumnType(column.elementTextTrim("DataType").substring(0, column.elementTextTrim("DataType").indexOf("(")));
                    } else {
                        column2.setColumnType(column.elementTextTrim("DataType"));
                    }

                    column2.setLength(column.elementTextTrim("Length") == null ? null : Integer.parseInt(column.elementTextTrim("Length")));

                    column2.setScale(column.elementTextTrim("Precision") == null ? null : Integer.parseInt(column.elementTextTrim("Precision")));

                    //字段说明
                    String colComment = column.elementText("Comment");
                    if (!(null == colComment || "".equals(colComment.trim()))) {

                        colComment = colComment.replaceAll("\r|\n", "");
                    }
                    column2.setColComment(colComment);

                    //查询主键编号
                    String pkID = column.attributeValue("Id");

                    if (table_s.element("Keys") != null) {

                        String keys_key_id = table_s.element("Keys").element("Key").attributeValue("Id");
                        String keys_column_ref = table_s.element("Keys").element("Key").element("Key.Columns")
                            .element("Column").attributeValue("Ref");
                        String keys_primarykey_ref_id = table_s.element("PrimaryKey").element("Key").attributeValue("Ref");

                        //判断是否为主键
                        if (keys_primarykey_ref_id.equals(keys_key_id) && keys_column_ref.equals(pkID)) {
                            column2.setPk(true);
                        }

                    }

                    columnList.add(column2);
                }
                table.setColumnList(columnList);

                tableList.add(table);
            }
        }

        return tableList;
    }

    public static long hash(String string) {
        long h = 1125899906842597L; // prime
        Random random = new Random();

        int number = random.nextInt(99);

        int len = string.length();

        for (int i = 0; i < len; i++) {
            h = number * h + string.charAt(i);
        }
        return h;
    }

//    public static void main(String[] args) {
//        XmlPrase xp = new XmlPrase();
//        List<Table> listTable = xp.getXmlTable("/develop/zy-master/zy-core/src/main/doc/pdm/core.pdm");
//
//        String javapath = "com/zy/core/backstage";
//        String htmlpath = "web/backstage";
//        String mapperpath = "resources/mapper";
//
//        CreateCodeFactory
//            .createcode("/Users/zhangyong/Desktop/test",listTable, javapath, htmlpath, mapperpath);
//
////        String tt = "/Users/zhangyong/Desktop/";
////
////        if(!tt.endsWith(File.separator)){
////            tt = tt + File.separator;
////        }
////        System.out.println("tt====" + tt);
////
////
////        long cc = -1367968409096863353L;
////        long ss = -3068837394742385883L;
////
////
////        Random random=new Random();
////
////        Integer number= random.nextInt(900000)+100000;
////
////        String numberString = System.currentTimeMillis()+String.valueOf(number);
////
////        System.out.println("--numberString====" + numberString);
////
////        System.out.println("--tt====" + hash(String.valueOf(numberString)));
////        System.out.println("--tt====" + hash(String.valueOf(numberString)));
////        System.out.println("--tt====" + KeyUtil.genUniqueKey());
//
////        String name = "TestStringName";
////        name = CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, name);
////
////        System.out.println("name====" + name);
////
////        name = CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.LOWER_UNDERSCORE, name);
////
////        System.out.println("name=22===" + name);
////
////        System.out.println(CaseFormat.LOWER_UNDERSCORE.to(CaseFormat.LOWER_CAMEL, "test_data"));
////
////        System.out.println(CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.UPPER_CAMEL, "test_data"));
//
//
//
////        for(int i = 0;i < 407;i++){
////            String randomDate = randomDate("2017-01-01 00:00:00", "2017-12-31 00:00:00");
////            System.out.println(randomDate);
////        }
//
//
//    }
    /**
     * 获取随机日期
     *
     * @param beginDate
     *            起始日期，格式为：yyyyMMdd
     * @param endDate
     *            结束日期，格式为：yyyyMMdd
     * @return
     */

    private static String randomDate(String beginDate, String endDate) {
        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date start = format.parse(beginDate);// 构造开始日期
            Date end = format.parse(endDate);// 构造结束日期
            // getTime()表示返回自 1970 年 1 月 1 日 00:00:00 GMT 以来此 Date 对象表示的毫秒数。
            if (start.getTime() >= end.getTime()) {
                return null;
            }
            long date = random(start.getTime(), end.getTime()) ;
            return format.format(new Date(date)) ;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private static long random(long begin, long end) {
        long rtn = begin + (long) (Math.random() * (end - begin));
        // 如果返回的是开始时间和结束时间，则递归调用本函数查找随机值
        if (rtn == begin || rtn == end) {
            return random(begin, end);
        }
        return rtn;
    }
}